@@ -211,7 +211,7 @@ class SOAP_WSDL extends SOAP_Base
      */
     function parseURL($wsdl_uri)
     {
-        $parser =& new $this->wsdlParserClass($wsdl_uri, $this, $this->docs);
+        $parser = new $this->wsdlParserClass($wsdl_uri, $this, $this->docs);
 
         if ($parser->fault) {
             $this->_raiseSoapFault($parser->fault);
@@ -331,6 +331,11 @@ class SOAP_WSDL extends SOAP_Base
             $opData['namespace'] = $this->bindings[$binding]['operations'][$operation]['input']['namespace'];
         // Message data from messages.
         $inputMsg = $opData['input']['message'];
+        if (isset($opData['input']['parts']) &&
+            !is_array($opData['input']['parts'])) {
+            $opData['input']['parts'] = array($opData['input']['parts'] => '');
+
+        }
         if (is_array($this->messages[$inputMsg])) {
             foreach ($this->messages[$inputMsg] as $pname => $pattrs) {
                 if ($opData['style'] == 'document' &&
@@ -350,6 +355,10 @@ class SOAP_WSDL extends SOAP_Base
             }
         }
         $outputMsg = $opData['output']['message'];
+        if (isset($opData['output']['parts']) &&
+            !is_array($opData['output']['parts'])) {
+            $opData['output']['parts'] = array($opData['output']['parts'] => '');
+        }
         if (is_array($this->messages[$outputMsg])) {
             foreach ($this->messages[$outputMsg] as $pname => $pattrs) {
                 if ($opData['style'] == 'document' &&
@@ -499,19 +508,25 @@ class SOAP_WSDL extends SOAP_Base
         return preg_match('/^[\w_:#\/]+$/', $string);
     }
 
-    function _addArg(&$args, &$argarray, $argname)
+    function _addArg(&$args, &$argarray, $argname, $nillable = false, &$nillableArgs = array())
     {
         if ($args) {
             $args .= ', ';
         }
         $args .= '$' . $argname;
+        if ($nillable) {
+            $args .= ' = null';
+            $nillableArgs[] = $argname;
+        }
         if (!$this->_validateString($argname)) {
             return;
         }
-        if ($argarray) {
-            $argarray .= ', ';
+        if (!$nillable) {
+            if ($argarray) {
+                $argarray .= ', ';
+            }
+            $argarray .= "'$argname' => $" . $argname;
         }
-        $argarray .= "'$argname' => $" . $argname;
     }
 
     function _elementArg(&$args, &$argarray, &$_argtype, $_argname)
@@ -675,7 +690,7 @@ class SOAP_WSDL extends SOAP_Base
                         $_argname = $this->_sanitize($_argname);
                         if ($opstyle == 'document' && $use == 'literal' &&
                             $_argtype['name'] == 'parameters') {
-                            // The type or element refered to is used for
+                            // The type or element referred to is used for
                             // parameters.
                             $elattrs = null;
                             $el = $this->elements[$_argtype['namespace']][$_argtype['type']];
@@ -685,20 +700,34 @@ class SOAP_WSDL extends SOAP_Base
                                 // XXX need to wrap the parameters in a
                                 // SOAP_Value.
                             }
+                            $usingNillables = false;
+                            $nillableArgs = array();
                             if (isset($el['elements'])) {
                                 foreach ($el['elements'] as $elname => $elattrs) {
                                     $elname = $this->_sanitize($elname);
+                                    if ((isset($elattrs['nillable']) && $elattrs['nillable'])
+                                    || (isset($elattrs['minOccurs']) && $elattrs['minOccurs'] == 0)) {
+                                        // If you encounter one nillable, all subsequent
+                                        // arguments must default to null
+                                        $usingNillables = true;
+                                    }
                                     // Is the element a complex type?
                                     if (isset($this->complexTypes[$elattrs['namespace']][$elname])) {
                                         $comments .= $this->_complexTypeArg($args, $argarray, $_argtype, $_argname);
                                     } else {
-                                        $this->_addArg($args, $argarray, $elname);
+                                        $this->_addArg($args, $argarray, $elname, $usingNillables, $nillableArgs);
                                     }
                                 }
                             }
                             if ($el['complex'] && $argarray) {
                                 $wrapname = '{' . $this->namespaces[$_argtype['namespace']].'}' . $el['name'];
-                                $comments .= "        \${$el['name']} = new SOAP_Value('$wrapname', false, \$v = array($argarray));\n";
+                                $comments .= "        \$v = array($argarray);\n";
+                                if ($usingNillables && !empty($nillableArgs)) {
+                                    foreach($nillableArgs as $nillableArg) {
+                                        $comments .= "        isset(\$$nillableArg) && \$v['$nillableArg'] = \$$nillableArg;\n";
+                                    }
+                                }
+                                $comments .= "        \${$el['name']} = new SOAP_Value('$wrapname', false, \$v);\n";
                                 $argarray = "'{$el['name']}' => \${$el['name']}";
                             }
                         } else {
@@ -788,7 +817,7 @@ class SOAP_WSDL extends SOAP_Base
             require_once 'SOAP/Client.php';
             eval($proxy);
         }
-        $proxy =& new $classname;
+        $proxy = new $classname;
 
         return $proxy;
     }
@@ -938,7 +967,7 @@ class SOAP_WSDL extends SOAP_Base
     {
         static $trail = array();
 
-        $arrayType = ereg_replace('\[\]$', '', $arrayType);
+        $arrayType = preg_replace('/\[\]$/', '', $arrayType);
 
         // Protect against circular references XXX We really need to remove
         // trail from this altogether (it's very inefficient and in the wrong
@@ -1156,7 +1185,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
     function SOAP_WSDL_Parser($uri, &$wsdl, $docs = false)
     {
         parent::SOAP_Base('WSDLPARSER');
-        $this->cache =& new SOAP_WSDL_Cache($wsdl->cacheUse,
+        $this->cache = new SOAP_WSDL_Cache($wsdl->cacheUse,
                                             $wsdl->cacheMaxAge,
                                             $wsdl->cacheDir);
         $this->uri = $uri;
@@ -1199,8 +1228,8 @@ class SOAP_WSDL_Parser extends SOAP_Base
     {
         // Get element prefix.
         $qname = new QName($name);
-        if ($qname->ns) {
-            $ns = $qname->ns;
+        if ($qname->prefix) {
+            $ns = $qname->prefix;
             if ($ns && ((!$this->tns && strcasecmp($qname->name, 'definitions') == 0) || $ns == $this->tns)) {
                 $name = $qname->name;
             }
@@ -1248,7 +1277,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
                     if (array_key_exists('base', $attrs)) {
                         $qn = new QName($attrs['base']);
                         $this->wsdl->complexTypes[$this->schema][$this->currentComplexType]['type'] = $qn->name;
-                        $this->wsdl->complexTypes[$this->schema][$this->currentComplexType]['namespace'] = $qn->ns;
+                        $this->wsdl->complexTypes[$this->schema][$this->currentComplexType]['namespace'] = $qn->prefix;
                     } else {
                         $this->wsdl->complexTypes[$this->schema][$this->currentComplexType]['type'] = 'Struct';
                     }
@@ -1262,8 +1291,8 @@ class SOAP_WSDL_Parser extends SOAP_Base
                 if (isset($attrs['type'])) {
                     $qn = new QName($attrs['type']);
                     $attrs['type'] = $qn->name;
-                    if ($qn->ns && array_key_exists($qn->ns, $this->wsdl->namespaces)) {
-                        $attrs['namespace'] = $qn->ns;
+                    if ($qn->prefix && array_key_exists($qn->prefix, $this->wsdl->namespaces)) {
+                        $attrs['namespace'] = $qn->prefix;
                     }
                 }
 
@@ -1361,7 +1390,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
                                     if ($q->name == 'arrayType') {
                                         $this->wsdl->complexTypes[$this->schema][$this->currentComplexType][$q->name] = $vq->name. $vq->arrayInfo;
                                         $this->wsdl->complexTypes[$this->schema][$this->currentComplexType]['type'] = 'Array';
-                                        $this->wsdl->complexTypes[$this->schema][$this->currentComplexType]['namespace'] = $vq->ns;
+                                        $this->wsdl->complexTypes[$this->schema][$this->currentComplexType]['namespace'] = $vq->prefix;
                                     } else {
                                         $this->wsdl->complexTypes[$this->schema][$this->currentComplexType][$q->name] = $vq->name;
                                     }
@@ -1388,7 +1417,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
                 }
                 if ($qn) {
                     $attrs['type'] = $qn->name;
-                    $attrs['namespace'] = $qn->ns;
+                    $attrs['namespace'] = $qn->prefix;
                 }
                 $this->wsdl->messages[$this->currentMessage][$attrs['name']] = $attrs;
                 // error in wsdl
@@ -1425,7 +1454,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
                     if (array_key_exists('message', $attrs)) {
                         $qn = new QName($attrs['message']);
                         $this->wsdl->portTypes[$this->currentPortType][$this->currentOperation][$name]['message'] = $qn->name;
-                        $this->wsdl->portTypes[$this->currentPortType][$this->currentOperation][$name]['namespace'] = $qn->ns;
+                        $this->wsdl->portTypes[$this->currentPortType][$this->currentOperation][$name]['namespace'] = $qn->prefix;
                     }
                 }
                 break;
@@ -1439,7 +1468,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
             break;
 
         case 'binding':
-            $ns = $qname->ns ? $this->wsdl->namespaces[$qname->ns] : SCHEMA_WSDL;
+            $ns = $qname->prefix ? $this->wsdl->namespaces[$qname->prefix] : SCHEMA_WSDL;
             switch ($ns) {
             case SCHEMA_SOAP:
             case SCHEMA_SOAP12:
@@ -1473,7 +1502,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
                     // soap:body attributes:
                     // part - optional.  listed parts must appear in body, missing means all parts appear in body
                     // use - required. encoded|literal
-                    // encodingStyle - optional.  space seperated list of encodings (uri's)
+                    // encodingStyle - optional.  space separated list of encodings (uri's)
                     $this->wsdl->bindings[$this->currentBinding]
                                     ['operations'][$this->currentOperation][$this->opStatus] = $attrs;
                     break;
@@ -1626,7 +1655,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
             break;
 
         case 'service':
-            $ns = $qname->ns ? $this->wsdl->namespaces[$qname->ns] : SCHEMA_WSDL;
+            $ns = $qname->prefix ? $this->wsdl->namespaces[$qname->prefix] : SCHEMA_WSDL;
 
             switch ($qname->name) {
             case 'port':
@@ -1636,13 +1665,13 @@ class SOAP_WSDL_Parser extends SOAP_Base
                 // XXX hack to deal with binding namespaces
                 $qn = new QName($attrs['binding']);
                 $this->wsdl->services[$this->currentService]['ports'][$this->currentPort]['binding'] = $qn->name;
-                $this->wsdl->services[$this->currentService]['ports'][$this->currentPort]['namespace'] = $qn->ns;
+                $this->wsdl->services[$this->currentService]['ports'][$this->currentPort]['namespace'] = $qn->prefix;
                 break;
 
             case 'address':
                 $this->wsdl->services[$this->currentService]['ports'][$this->currentPort]['address'] = $attrs;
                 // what TYPE of port is it?  SOAP or HTTP?
-                $ns = $qname->ns ? $this->wsdl->namespaces[$qname->ns] : SCHEMA_WSDL;
+                $ns = $qname->prefix ? $this->wsdl->namespaces[$qname->prefix] : SCHEMA_WSDL;
                 switch ($ns) {
                 case SCHEMA_WSDL_HTTP:
                     $this->wsdl->services[$this->currentService]['ports'][$this->currentPort]['type']='http';
@@ -1670,27 +1699,31 @@ class SOAP_WSDL_Parser extends SOAP_Base
         // Top level elements found under wsdl:definitions.
         switch ($qname->name) {
         case 'import':
-            // sect 2.1.1 wsdl:import attributes: namespace location
-            if ((isset($attrs['location']) || isset($attrs['schemaLocation'])) &&
-                !isset($this->wsdl->imports[$attrs['namespace']])) {
+        case 'include':
+            // WSDL 2.1.1 wsdl:import, XML Schema 4.2.3 xsd:import, XML Schema
+            // 4.2.1 xsd:include attributes
+            $this->status = 'types';
+            if (isset($attrs['location']) || isset($attrs['schemaLocation'])) {
                 $uri = isset($attrs['location']) ? $attrs['location'] : $attrs['schemaLocation'];
                 $location = @parse_url($uri);
                 if (!isset($location['scheme'])) {
                     $base = @parse_url($this->uri);
                     $uri = $this->mergeUrl($base, $uri);
                 }
+                if (isset($this->wsdl->imports[$uri])) {
+                    break;
+                }
+                $this->wsdl->imports[$uri] = $attrs;
 
-                $this->wsdl->imports[$attrs['namespace']] = $attrs;
                 $import_parser_class = get_class($this);
-                $import_parser =& new $import_parser_class($uri, $this->wsdl, $this->docs);
+                $import_parser = new $import_parser_class($uri, $this->wsdl, $this->docs);
                 if ($import_parser->fault) {
-                    unset($this->wsdl->imports[$attrs['namespace']]);
+                    unset($this->wsdl->imports[$uri]);
                     return false;
                 }
-                $this->currentImport = $attrs['namespace'];
             }
-            // Continue on to the 'types' case - lack of break; is
-            // intentional.
+            $this->status = 'types';
+            break;
 
         case 'types':
             // sect 2.2 wsdl:types
@@ -1732,14 +1765,14 @@ class SOAP_WSDL_Parser extends SOAP_Base
         case 'binding':
             // sect 2.5 wsdl:binding attributes: name type
             // children: wsdl:operation soap:binding http:binding
-            if ($qname->ns && $qname->ns != $this->tns) {
+            if ($qname->prefix && $qname->prefix != $this->tns) {
                 break;
             }
             $this->status = 'binding';
             $this->currentBinding = $attrs['name'];
             $qn = new QName($attrs['type']);
             $this->wsdl->bindings[$this->currentBinding]['type'] = $qn->name;
-            $this->wsdl->bindings[$this->currentBinding]['namespace'] = $qn->ns;
+            $this->wsdl->bindings[$this->currentBinding]['namespace'] = $qn->prefix;
             break;
 
         case 'service':
